local bit = bit32 or require 'bit'
local crc_table

local function make_crc_table()
  crc_table = {}
  local poly = 0xedb88320
  for n= 0,255 do
    local c = n
    for k=1,8 do
        c = bit.band(c,1) ~= 0 and
          bit.bxor(poly, bit.rshift(c,1)) or bit.rshift(c,1)
    end
    crc_table[n] = c
  end
end

local function crc32(data, crc)
  crc = bit.bnot(crc or 0)
  if not crc_table then make_crc_table() end
  local bxor, band, rshift = bit.bxor, bit.band, bit.rshift
  for i = 1, #data do
    crc = bxor(crc_table[band(bxor(crc, data:sub(i,i):byte()),
      0xff)], rshift(crc, 8))
  end
  return bit.bnot(crc)
end

-- Test
local msg = ("0123456789"):rep(100000)
local t = os.clock()
local a
for i=1,10 do
  a = crc32(msg, a)
end
print(string.format("%X", a))
print(os.clock()-t)

